/*
 * Copyright (c) 2020. Synclovis Systems Pvt Ltd - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */

package com.synclovis.trackflow.connectors;


import android.os.Build;

import androidx.annotation.RequiresApi;

import com.github.nkzawa.emitter.Emitter;
import com.github.nkzawa.socketio.client.IO;
import com.github.nkzawa.socketio.client.Socket;
import com.synclovis.mytrackflowlibrary.utils.LibConstants;
import com.synclovis.mytrackflowlibrary.utils.LibTFUtils;
import com.synclovis.trackflow.utils.Constants;
import com.synclovis.trackflow.utils.TFUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.net.URLEncoder;

/**
 * Client - WebSocket Singleton class
 *
 * @author Anooj Krishnan G <anooj.krishnan@synclovis.com>
 * @author Nousheen K <nousheen.k@synclovis.com>
 * @version 1.0
 */


public class Client {

  private static Client instance;
  private Socket mSocket;
  private SocketListener socketListener;
  private String mApiKey = "";
  JSONObject requestFromUserID = null;

  public synchronized static Client getInstance() {
    if (instance == null) {
      instance = new Client();
    }
    return instance;
  }


  @RequiresApi(api = Build.VERSION_CODES.O)
  public void connectSocket(String apiKey, JSONObject metaInfo, SocketListener socketListener) {
    try {

      this.socketListener = socketListener;
      if (mSocket == null) {
        mApiKey = apiKey;
        IO.Options mOptions = new IO.Options();
        String packageName = metaInfo.getString(Constants.PACKAGE_NAME);
        String deviceToken = metaInfo.getString(Constants.DEVICE_IDENTIFIER);
        deviceToken=deviceToken.trim();
        mOptions.query = "apiKey="+URLEncoder.encode(apiKey,"utf-8")+"&pName="+ URLEncoder.encode(packageName,"utf-8")+"&dId="+URLEncoder.encode(deviceToken,"utf-8")+ "&platform="+"REACT_NATIVE"+"&type="+"MOBILE";
        mSocket = IO.socket(Constants.BASE_URL, mOptions);
        // LibTFUtils.showLog("DEVICETOKEN---->"+deviceToken);

        listenEvents(apiKey);
        mSocket.connect();
      } else if (!mSocket.connected()) {
        mSocket.connect();
      }


    } catch (Exception e) {
      e.printStackTrace();
    }
  }


  public void listenEvents(String apiKey) {
    mSocket.on(Events.ON_CONNECTION, onConnection);
    mSocket.on(Events.ON_SOCKET_ERROR, onSocket_error);
    mSocket.on(Events.ON_ERROR, onError);
    mSocket.on(Events.ON_ACK,onAcknowledge);

    mSocket.on(Events.ON_SCREENSHOT_REQ, onCaptureScreenshot);
    mSocket.on(Events.ON_PERMISSION,onPermission);
    mSocket.on(Events.MEMBER_ONLINE,onMembersOnline);
    mSocket.on(Events.DEV_ASYNC_REQ,onAsyncData);
    mSocket.on(Events.DEV_INFO_REQ,onDeviceInfo);



  }

  private Emitter.Listener onConnection = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null) {
        socketListener.onConnectionSuccess();
      }
    }
  };


  private Emitter.Listener onSocket_error = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null) {

      }
    }
  };

  private Emitter.Listener onError = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null) {

      }
    }
  };

  private Emitter.Listener onAcknowledge = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null) {
      }
    }
  };

  private Emitter.Listener onCaptureScreenshot = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      try {
        requestFromUserID = new JSONObject(args[0].toString());
        if (socketListener != null  && LibConstants.SCREENSHOT_SERVICE) {
          socketListener.onScreenShotRequest();
        }else{
          serviceDisabled("screenShot_request");
        }
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
  };




  private Emitter.Listener onPermission = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null && LibConstants.PERMISSION_SERVICE) {
        socketListener.onPermissionRequest();
      }else{
        serviceDisabled("PERMISSION_SERVICE");
      }
    }
  };

  private Emitter.Listener onMembersOnline = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null) {

      }
    }
  };
  private Emitter.Listener onAsyncData = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null && LibConstants.SHAREDPREF_SERVICE) {
        socketListener.onAsyncRequest();
      }else{
        serviceDisabled("SHAREDPREF_SERVICE");
      }
    }
  };

  private Emitter.Listener onDeviceInfo = new Emitter.Listener() {
    @Override
    public void call(final Object... args) {
      if (socketListener != null && LibConstants.DEVICEINFO_SERVICE) {
        socketListener.onDeviceInfo();
      }else{
        serviceDisabled("DEVICEINFO_SERVICE");
      }
    }
  };



  public void sendPermission(JSONObject permissions) {
    if (mSocket != null) {
      mSocket.emit(Events.PERMISSION_LIST, permissions);
    }
  }

  public void sendScreenshot(String base64,String screenName) {
    // LibTFUtils.showLog("screenName---->"+screenName);
    if (mSocket != null && LibConstants.SCREENSHOT_SERVICE) {
      try {
        mSocket.emit(Events.ON_SCREENSHOT, base64, requestFromUserID.getString("requestFrom"),screenName);
      } catch (JSONException e) {
        e.printStackTrace();
      }
    }
  }
  public void screenNavigation(String base64,String screenName) {
    if (mSocket != null  && mSocket.connected() && LibConstants.SCREEN_NAVIGATION) {
      mSocket.emit(Events.ON_SCREENSHOT_AUTOMATIC, base64,screenName);
    }
  }

  public void shakeDevice(String base64,String screenName) {
    if (mSocket != null  && mSocket.connected()  && LibConstants.SHAKE_SERVICE) {
      // LibTFUtils.showLog("SHAKEDEVICE---->"+screenName);
     // LibTFUtils.showLog("SHAKEDEVICESCREENSHOT---->"+base64);


      mSocket.emit(Events.ON_SCREENSHOT_SHAKE, base64,screenName);
    }
  }

  public void sendNetworkLogs(String logs) {
    if (mSocket != null && mSocket.connected() && LibConstants.NETWORK_SERVICE) {
      mSocket.emit(Events.EMIT_LOG, logs);
      if (socketListener != null && LibConstants.SCREEN_NAVIGATION) {
        socketListener.onApiCall();
      }
    }
    else{
      serviceDisabled("NETWORK_SERVICE");
    }
  }

  //send AsyncStorage data
  public void sendAsyncLogs(String asynclogs) {

    if (mSocket != null && mSocket.connected()) {
      mSocket.emit(Events.DEV_ASYNC_RES, asynclogs);
    }
  }


  public void deviceInfo(JSONObject deviceDetails) {
    if (mSocket != null) {
      mSocket.emit(Events.EMIT_DEVICE_INIT, deviceDetails);
    }
  }
  public void deviceDetail(JSONObject deviceDetails) {
    if (mSocket != null) {
      mSocket.emit(Events.DEV_INFO_RES, deviceDetails);
    }
  }
  public void serviceDisabled(String request) {
   // LibTFUtils.showLog("Service disabled---->"+request);
     /*   JSONObject serviceDisabled = new JSONObject();
        try {
            //LibTFUtils.showLog("permission not granted");
            serviceDisabled.put("serviceDisabled", "please enable the service");
            mSocket.emit(Events.ON_SERVICE_DISABLED, serviceDisabled);
        }
        catch (JSONException e) {
            e.printStackTrace();
        }*/
  }

  public void sendLog(String log) {
    if (mSocket != null) {
      mSocket.emit(Events.CUSTOM_LOG,log );
    }
  }
}
